@@ -1,9 +1,11 @@
 import warnings
+import functools
 from ._src import allreduce, bcast, send, recv, sendrecv
 
 
 def deprecated_new_name(message):
     def deprecated_decorator(func):
+        @functools.wraps(func)
         def deprecated_func(*args, **kwargs):
             warnings.warn(
                 """{} has been renamed to {} and deprecated.
@@ -11,10 +13,8 @@ def deprecated_new_name(message):
                 """.format(
                     func.__name__, message
                 ),
-                category=DeprecationWarning,
                 stacklevel=2,
             )
-            warnings.simplefilter("default", DeprecationWarning)
             return func(*args, **kwargs)
 
         return deprecated_func
@@ -32,16 +32,16 @@ def Bcast(*args, **kwargs):
     return bcast(*args, **kwargs)
 
 
-@deprecated_new_name("send")
-def Send(*args, **kwargs):
-    return send(*args, **kwargs)
-
-
 @deprecated_new_name("recv")
 def Recv(*args, **kwargs):
     return recv(*args, **kwargs)
 
 
+@deprecated_new_name("send")
+def Send(*args, **kwargs):
+    return send(*args, **kwargs)
+
+
 @deprecated_new_name("sendrecv")
 def Sendrecv(*args, **kwargs):
     return sendrecv(*args, **kwargs)
